// Project #26 – Building a PC-Controlled Relay Board

#define latch 8 // latch RCK pin
#define clock 9 // clock SRCK pin
#define data 10 // data SERIN pin

int relayStatus; 

void showStatus()
{
  Serial.print("Relay status 7 to 0 is: ");
  Serial.println(relayStatus,BIN);
}

void waveHello()
{
  int d = 250;
  Serial.println("Hello!");
  allOff();
  for (int a=0; a<8; a++)
  {
    relayOn(a);
    delay(d);
    relayOff(a);
  }
  for (int a=6; a>=0; a--)
  {
    relayOn(a);
    delay(d);
    relayOff(a);
  }  
}

void relayOn(int a)
{
  relayStatus = relayStatus|(1<<a);
  sendStatus(relayStatus);
  Serial.print("Relay "); Serial.print(a); Serial.println(" On");
}

void relayOff(int a)
{
  relayStatus = relayStatus^(1<<a);
  sendStatus(relayStatus);
  Serial.print("Relay "); Serial.print(a); Serial.println(" Off");  
}

void allOn()
{
  relayStatus = 255;
  sendStatus(relayStatus);
  Serial.println("All relays turned on");   
}

void allOff()
{
  relayStatus = 0;
  sendStatus(relayStatus);
  Serial.println("All relays turned off");   
}

void sendStatus(int a)
{
  digitalWrite(latch, LOW);
  shiftOut(data, clock, MSBFIRST, a);
  digitalWrite(latch, HIGH);
}

void setup()
{
  Serial.begin(9600);
  pinMode(latch, OUTPUT);
  pinMode(clock, OUTPUT);
  pinMode(data, OUTPUT);
}

void loop()
{
  char a = 0;
  Serial.flush();
  while (Serial.available() == 0) {}
  while (Serial.available() > 0)
  {
    a = Serial.read();
  }
  switch (a)
  {
    case '0' : relayOn(0); break;
    case '1' : relayOn(1); break; 
    case '2' : relayOn(2); break;         
    case '3' : relayOn(3); break;     
    case '4' : relayOn(4); break;     
    case '5' : relayOn(5); break;     
    case '6' : relayOn(6); break; 
    case '7' : relayOn(7); break;         
    case '8' : relayOff(0); break; 
    case '9' : relayOff(1); break; 
    case 'A' : relayOff(2); break; 
    case 'B' : relayOff(3); break;           
    case 'C' : relayOff(4); break; 
    case 'D' : relayOff(5); break; 
    case 'E' : relayOff(6); break; 
    case 'F' : relayOff(7); break; 
    case 'G' : allOn(); break; 
    case 'H' : allOff(); break; 
    case 'Z' : waveHello(); break;     
    case '?' : showStatus(); break;           
    default : Serial.print("Incorrect command - 0 1 2 3 4 5 6 7 turns 0~7 on, ");
        Serial.println("8 9 A B C D E F turns 0~7 off, G - all on, H - all off");

  }
}