// Project #31 - Creating a Temperature and Air Pressure Display with Multiple Buses
#include <U8g2lib.h>
#include <Adafruit_BMP085.h>
#include <Wire.h>

U8G2_SSD1306_128X32_UNIVISION_F_HW_I2C u8g2(U8G2_R0, U8X8_PIN_NONE);
Adafruit_BMP085 bmp;
float temperature;
float pressure;

void TCA9548A(uint8_t bus)
{
  Wire.beginTransmission(0x70);  // TCA9548A address is 0x70
  Wire.write(1 << bus);          // send byte to select bus
  Wire.endTransmission();
}

void setup()
{
  Wire.begin();
  u8g2.begin();
  u8g2.setFont(u8g2_font_logisoso32_tr);  // select font
  TCA9548A(0);    // select I2C bus 0 for the BMP180
  bmp.begin();
}

void loop()
{
  // first, get the temperature from the BMP180
  TCA9548A(0);  // select I2C bus 0 for the BMP180
  temperature = bmp.readTemperature();
  pressure = bmp.readPressure();

  // next, display the temperature on the OLED
  TCA9548A(1);  // select I2C bus 1 for the OLED
  u8g2.clearBuffer();  // clear OLED memory
  u8g2.setCursor(0, 32);
  u8g2.print(int(temperature));
  u8g2.drawStr(50, 32, "C");
  u8g2.sendBuffer();
  delay(1000);

  u8g2.clearBuffer();   // clear OLED memory
  u8g2.setCursor(0, 32);
  pressure = pressure / 100;  // convert to hectopascals
  u8g2.print(int(pressure));
  u8g2.drawStr(80, 32, "h");
  u8g2.sendBuffer();
  delay(1000);
}
