// Project 44 - Creating a Low-Power Logger for Periodic Data

#include <SD.h>
#include <LowPower.h>
#include <DS3231.h>
#include <Wire.h>
#include <Adafruit_BMP085.h>

Adafruit_BMP085 bmp;
DS3231 RTCmodule;

bool century = false;
bool h12Flag = false;
bool pmFlag = false;

int hh, mm, ss, dd, mo, yy, temperature, pressure;

void setup()
{
  delay(3000); // Allow time for SD card to be inserted.
  Serial.begin(9600);
  Serial.println("Initializing SD card...");
  pinMode(10, OUTPUT);
  // check that the memory card exists and is usable
  if (!SD.begin(10))
  {
    Serial.println("Card failed, or not present");
    // stop sketch
    return;
  }
  Serial.println("memory card is ready");
  Wire.begin();
  bmp.begin();
}

void setTime()
{
  RTCmodule.setClockMode(false);  // Set to 24h time.
  RTCmodule.setYear(2022);
  RTCmodule.setMonth(1);
  RTCmodule.setDate(11);
  RTCmodule.setDoW(2);
  RTCmodule.setHour(17);
  RTCmodule.setMinute(11);
  RTCmodule.setSecond(8);
}

void logData()
{
  // create the file for writing
  File dataFile = SD.open("DATA.TXT", FILE_WRITE);
  // if the file is ready, write to it:
  if (dataFile)
  {
    // Get data from RTC.
    dd = RTCmodule.getDate();
    mo = RTCmodule.getMonth(century);
    yy = RTCmodule.getYear();
    hh = RTCmodule.getHour(h12Flag, pmFlag);
    mm = RTCmodule.getMinute();
    ss = RTCmodule.getSecond();
        
    // Write time and date to SD card
    dataFile.print(dd, DEC);
    dataFile.print("/");
    dataFile.print(mo, DEC);
    dataFile.print("/");
    dataFile.print(yy, DEC);
    dataFile.print(" | ");    
    dataFile.print(hh, DEC);
    dataFile.print(":");
    if (mm < 10) // Check for leading zero on minutes.
    {
      dataFile.print("0");
    }
    dataFile.print(mm, DEC);
    dataFile.print(":");
    if (ss < 10) // Check for leading zero on seconds.
    {
      dataFile.print("0");
    }
    dataFile.print(ss, DEC);
    dataFile.print(" | ");

    // write temperature data
    temperature = bmp.readTemperature();
    pressure = int(bmp.readPressure() / 100);
    dataFile.print(temperature, DEC);
    dataFile.print(" C - ");
    dataFile.print(pressure, DEC);
    dataFile.println(" hPa");
    dataFile.close(); // close the file once the system has finished with it
  }
}

void loop()
{
  // setTime(); // Used to set time in RTC if necessary.
  logData();
  for (int snooze = 0; snooze<8; snooze++)
  // Idle for 64 seconds.
  {
    LowPower.powerDown(SLEEP_8S, ADC_OFF, BOD_OFF);
  }
}
