// Project 45 - Triggering Data Logging with Low Power Using Interrupts

#include <SD.h>
#include <LowPower.h>
#include <DS3231.h>
#include <Wire.h>
DS3231 RTCmodule;

bool century = false;
bool h12Flag = false;
bool pmFlag = false;

int hh, mm, ss, dd, mo, yy;

void setup()
{
  delay(3000); // Allow time for SD card to be inserted.
  Serial.begin(9600);
  Serial.println("Initializing SD card...");
  pinMode(10, OUTPUT);
  // Check that the memory card exists and is usable.
  if (!SD.begin(10))
  {
    Serial.println("Card failed, or not present.");
    // stop sketch
    return;
  }
  Serial.println("Memory card is ready!");
  Wire.begin();
  pinMode(13, OUTPUT);
}

void interruptFunction() {}

void setTimeData()
// Use once to set the time and date in the RTC module.
{
  RTCmodule.setClockMode(false);  // Set to 24h time.
  RTCmodule.setYear(2022);
  RTCmodule.setMonth(1);
  RTCmodule.setDate(11);
  RTCmodule.setDoW(2);
  RTCmodule.setHour(17);
  RTCmodule.setMinute(11);
  RTCmodule.setSecond(8);
}

void logData()
{
  // create the file for writing
  File dataFile = SD.open("DATA.TXT", FILE_WRITE);
  // if the file is ready, write to it:
  if (dataFile)
  {
    // Get data from RTC.
    dd = RTCmodule.getDate();
    mo = RTCmodule.getMonth(century);
    yy = RTCmodule.getYear();
    hh = RTCmodule.getHour(h12Flag, pmFlag);
    mm = RTCmodule.getMinute();
    ss = RTCmodule.getSecond();
        
    // Write time and date to SD card
    dataFile.print(dd, DEC);
    dataFile.print("/");
    dataFile.print(mo, DEC);
    dataFile.print("/");
    dataFile.print(yy, DEC);
    dataFile.print(" | ");    
    dataFile.print(hh, DEC);
    dataFile.print(":");
    if (mm < 10) // Check for leading zero on minutes.
    {
      dataFile.print("0");
    }
    dataFile.print(mm, DEC);
    dataFile.print(":");
    if (ss < 10) // Check for leading zero on seconds.
    {
      dataFile.print("0");
    }
    dataFile.print(ss, DEC);
    dataFile.println(".");
    dataFile.close(); // Close the file once the system has finished with it    
  }
}

void loop()
{
  attachInterrupt(0, interruptFunction, RISING);
  LowPower.powerDown(SLEEP_FOREVER, ADC_OFF, BOD_OFF);
  detachInterrupt(0);
  logData();
}
