// Project 55 - Controlling a Single-GPIO Remotely

#include <WiFi.h>
WiFiServer server(80); // Set webserver port to 80

// store your WiFi network credentials
char* ssid = "SSID";
char* password = "password";

String request; // Stores HTTP request from client (web browser)
int outputPin = 16; // Using GPIO pin 16 on the ESP32 board

unsigned long currentTime = millis();
unsigned long previousTime = 0; 
const long timeoutTime = 2000;  // Allowed client connection time

void setup()
{
  Serial.begin(115200);
  // Setup digital pin to control
  pinMode(outputPin, OUTPUT);
  digitalWrite(outputPin, LOW);

  // Connect to Wi-Fi network
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED)
  // Show indicator while waiting for connection
  {
    delay(500);
    Serial.print("~");
  }
  Serial.println(); // Display local IP address on serial monitor
  Serial.print("Connected, IP address: ");
  Serial.println(WiFi.localIP());
  //  Start web server
  server.begin();
}

void loop() 
{
  // Listen for incoming clients fromn web browser
  WiFiClient client = server.available();   
  if (client) // If a request received
  {
    currentTime = millis();
    previousTime = currentTime;
    Serial.println("New client connected");       
    String currentLine = ""; 
    while (client.connected() && currentTime - previousTime <= timeoutTime) 
    {  // Stay connected for timeoutTime
      currentTime = millis();
      if (client.available()) 
      {
        char c = client.read(); // Display data from client on Serial Monitor
        Serial.write(c); 
        request += c;
        if (c == '\n') // Client request has finished
        {                    
          if (currentLine.length() == 0) 
          {
            // Send HTTP reponse back to client
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();
            // Now do things based on the client request
            if (request.indexOf("GET /16/on") >= 0) 
            {
              // If request was IP address/16/on
              Serial.println("output on");
              digitalWrite(outputPin, HIGH);
            } else if (request.indexOf("GET /16/off") >= 0) 
            // If request was IP address/16/off
            {
              Serial.println("output off");
              digitalWrite(outputPin, LOW);
            } 

            // Build web page for display on browser
            client.println("<!DOCTYPE html><html>");
            client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
            client.println("<link rel=\"icon\" href=\"data:,\">");
            // Build buttons using CSS
            client.println("<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}");
            client.println(".button { background-color: #ff0000; border: none; color: white; padding: 8px 20px;"); 
            client.println("text-decoration: none; font-size: 20px; margin: 2px; cursor: pointer; border-radius: 12px;}");
            client.println("</style></head>");
            // Now HTML for layout
            client.println("<body><h3>Remote ESP32 Output Control</h3>");
            client.println("<p><a href=\"/16/on\"><button class=\"button\">On</button></a>"); // Display "On" button with matching HTML "/16/on"
            client.println("<a href=\"/16/off\"><button class=\"button\">Off</button></a></p>"); // Display "Off" button with matching HTML "/16/off"
            client.println("</body></html>");
            // End the response to client request            
            client.println();
            break;
          } else { // if you got a newline, then clear currentLine
            currentLine = "";
          }
        } else if (c != '\r') 
        { 
          currentLine += c;
        }
      }
    }
    request = "";
    // Close connection to client
    client.stop();
    Serial.println("Disconnected.");
  }
}
