// Project 56 - Creating a Four-GPIO Remote Control with Status Display

#include <WiFi.h>
WiFiServer server(80); // Set webserver port to 80

// store your WiFi network credentials
char* ssid = "SSID";
char* password = "password";

String request; // Stores HTTP request from client (web browser)
String outputPin1State = "Off";
String outputPin2State = "Off";
String outputPin3State = "Off";
String outputPin4State = "Off";

int outputPin1 = 16; // Using GPIO pins 16~19 on the ESP32 board
int outputPin2 = 17;
int outputPin3 = 18;
int outputPin4 = 19;

unsigned long currentTime = millis();
unsigned long previousTime = 0; 
const long timeoutTime = 2000;  // Allowed client connection time

void setup()
{
  Serial.begin(115200);
  // Setup digital pins to control
  pinMode(outputPin1, OUTPUT);
  pinMode(outputPin2, OUTPUT);
  pinMode(outputPin3, OUTPUT);
  pinMode(outputPin4, OUTPUT);  
  digitalWrite(outputPin1, LOW);
  digitalWrite(outputPin2, LOW);  
  digitalWrite(outputPin3, LOW);
  digitalWrite(outputPin4, LOW);  
  // Connect to Wi-Fi network
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED)
  // Show indicator while waiting for connection
  {
    delay(500);
    Serial.print("~");
  }
  Serial.println(); // Display local IP address on serial monitor
  Serial.print("Connected, IP address: ");
  Serial.println(WiFi.localIP());
  //  Start web server
  server.begin();
}

void loop() 
{
  // Listen for incoming clients fromn web browser
  WiFiClient client = server.available();   
  if (client) // If a request received
  {
    currentTime = millis();
    previousTime = currentTime;
    Serial.println("New client connected");       
    String currentLine = ""; 
    while (client.connected() && currentTime - previousTime <= timeoutTime) 
    {  // Stay connected for timeoutTime
      currentTime = millis();
      if (client.available()) 
      {
        char c = client.read(); // Display data from client on Serial Monitor
        Serial.write(c); 
        request += c;
        if (c == '\n') // Client request has finished
        {                    
          if (currentLine.length() == 0) 
          {
            // Send HTTP reponse back to client
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();
            
            // Now do things based on the client request
            if (request.indexOf("GET /16/on") >= 0) 
            {
              // If request was IP address/16/on
              Serial.println("output 1 on");
              outputPin1State = "On";
              digitalWrite(outputPin1, HIGH);
            } else if (request.indexOf("GET /16/off") >= 0) 
            // If request was IP address/16/off
            {
              Serial.println("output 1 off");
              outputPin1State = "Off";
              digitalWrite(outputPin1, LOW);
            } else if (request.indexOf("GET /17/on") >= 0) 
            {
              // If request was IP address/17/on
              Serial.println("output 2 on");
              outputPin2State = "On";
              digitalWrite(outputPin2, HIGH);
            } else if (request.indexOf("GET /17/off") >= 0) 
            // If request was IP address/17/off
            {
              Serial.println("output 2 off");
              outputPin2State = "Off";
              digitalWrite(outputPin2, LOW);
            } 
             else if (request.indexOf("GET /18/on") >= 0) 
            {
              // If request was IP address/18/on
              Serial.println("output 3 on");
              outputPin3State = "On";
              digitalWrite(outputPin3, HIGH);
            } else if (request.indexOf("GET /18/off") >= 0) 
            // If request was IP address/18/off
            {
              Serial.println("output 3 off");
              outputPin3State = "Off";
              digitalWrite(outputPin3, LOW);
            } 
             else if (request.indexOf("GET /19/on") >= 0) 
            {
              // If request was IP address/19/on
              Serial.println("output 4 on");
              outputPin4State = "On";
              digitalWrite(outputPin4, HIGH);
            } else if (request.indexOf("GET /19/off") >= 0) 
            // If request was IP address/19/off
            {
              outputPin4State = "Off";
              Serial.println("output 4 off");
              digitalWrite(outputPin4, LOW);
            } 

            // Build web page for display on browser
            client.println("<!DOCTYPE html><html>");
            client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
            client.println("<link rel=\"icon\" href=\"data:,\">");
            // Build buttons using CSS
            client.println("<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}");
            client.println(".button { background-color: #ff0000; border: none; color: white; padding: 8px 20px;"); 
            client.println("text-decoration: none; font-size: 20px; margin: 2px; cursor: pointer; border-radius: 12px;}");
            client.println("</style></head>");
            // Now HTML for layout
            client.println("<body><h3>Remote ESP32 Output Controls</h3>");
            client.println("<p><b>Output 1 is "); 
            client.println(outputPin1State);
            client.println(" - </b><a href=\"/16/on\"><button class=\"button\">On</button></a>");
            client.println("<a href=\"/16/off\"><button class=\"button\">Off</button></a></p>");

            client.println("<p><b>Output 2 is "); 
            client.println(outputPin2State);
            client.println(" - </b><a href=\"/17/on\"><button class=\"button\">On</button></a>");
            client.println("<a href=\"/17/off\"><button class=\"button\">Off</button></a></p>");

            client.println("<p><b>Output 3 is "); 
            client.println(outputPin3State);
            client.println(" - </b><a href=\"/18/on\"><button class=\"button\">On</button></a>");
            client.println("<a href=\"/18/off\"><button class=\"button\">Off</button></a></p>");

            client.println("<p><b>Output 4 is "); 
            client.println(outputPin4State);
            client.println(" - </b><a href=\"/19/on\"><button class=\"button\">On</button></a>");
            client.println("<a href=\"/19/off\"><button class=\"button\">Off</button></a></p>");                        
            
            client.println("</body></html>");
            // End the response to client request            
            client.println();
            break;
          } else { // if you got a newline, then clear currentLine
            currentLine = "";
          }
        } else if (c != '\r') 
        { 
          currentLine += c;
        }
      }
    }
    request = "";
    // Close connection to client
    client.stop();
    Serial.println("Disconnected.");
  }
}
