// Project 57 - Using a GPIO Remote Control with PWM

#include <WiFi.h>
WiFiServer server(80); // Set webserver port to 80

// store your WiFi network credentials
char* ssid = "SSID";
char* password = "password";

int outputPin1 = 16;
int dutyCycle = 0;
String request; // Stores HTTP request from client (web browser)

unsigned long currentTime = millis();
unsigned long previousTime = 0;
const long timeoutTime = 2000;  // Allowed client connection time


void pinOff()
{
  ledcDetachPin(16); // disconnect from PWM
  digitalWrite(16, LOW);  // pin off
}

void pinOn()
{
  ledcDetachPin(16); // disconnect from PWM
  digitalWrite(16, HIGH); // usual turn pin on
}

void pinPWM(int dc)
{
  // Setup digital pin for PWM
  ledcSetup(0, 5000, 8); // LED channel, frequency, resolution (8-bit)
  ledcAttachPin(16, 0);  // Attach PWM - GPIO, LED channel
  ledcWrite(0, dc);      // Set duty cycle
}

void setup()
{
  Serial.begin(115200);
  pinMode(16, OUTPUT);
  pinOff();

  // Connect to Wi-Fi network
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED)
    // Show indicator while waiting for connection
  {
    delay(500);
    Serial.print("~");
  }
  Serial.println(); // Display local IP address on serial monitor
  Serial.print("Connected, IP address: ");
  Serial.println(WiFi.localIP());
  //  Start web server
  server.begin();
}

void loop()
{
  // Listen for incoming clients fromn web browser
  WiFiClient client = server.available();
  if (client) // If a request received
  {
    currentTime = millis();
    previousTime = currentTime;
    Serial.println("New client connected");
    String currentLine = "";
    while (client.connected() && currentTime - previousTime <= timeoutTime)
    { // Stay connected for timeoutTime
      currentTime = millis();
      if (client.available())
      {
        char c = client.read(); // Display data from client on Serial Monitor
        Serial.write(c);
        request += c;
        if (c == '\n') // Client request has finished
        {
          if (currentLine.length() == 0)
          {
            // Send HTTP reponse back to client
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();

            // Now do things based on the client request
            if (request.indexOf("GET /off") >= 0)
            {
              // If request was IP address/off
              pinOff();
              dutyCycle = 0;
            } else if (request.indexOf("GET /dcdown") >= 0)
              // If request was reduce duty cycle
            {
              if (dutyCycle >= 2)
              {
                pinPWM(dutyCycle - 1);
                dutyCycle--;
              } else if (dutyCycle <= 1)
              {
                pinOff();
                dutyCycle = 0;
              }
            } else if (request.indexOf("GET /dcup") >= 0)
            {
              // If request was increase duty cycle
              if (dutyCycle > 253)
              {
                pinOn();
                dutyCycle = 255;
              }
              else if (dutyCycle <= 253)
              {
                pinPWM(dutyCycle + 1);
                dutyCycle++;
              }

            } else if (request.indexOf("GET /on") >= 0)
              // // If request was IP address/on
            {
              pinOn();
              dutyCycle = 255;
            }
            else if (request.indexOf("GET /35") >= 0)
            {
              // If request was IP address/35
              pinPWM(35);
              dutyCycle = 35;
            } else if (request.indexOf("GET /50") >= 0)
              // If request was IP address/50
            {
              pinPWM(50);
              dutyCycle = 50;
            }
            else if (request.indexOf("GET /65") >= 0)
            {
              // If request was IP address/65
              pinPWM(65);
              dutyCycle = 65;
            } else if (request.indexOf("GET /75") >= 0)
              // If request was IP address/75
            {
              pinPWM(75);
              dutyCycle = 75;
            }

            // Build web page for display on browser
            client.println("<!DOCTYPE html><html>");
            client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
            client.println("<link rel=\"icon\" href=\"data:,\">");
            // Build buttons using CSS
            client.println("<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}");
            client.println(".button { background-color: #ff0000; border: none; color: white; padding: 8px 20px;");
            client.println("text-decoration: none; font-size: 20px; margin: 2px; cursor: pointer; border-radius: 12px;}");
            client.println("</style></head>");
            // Now HTML for layout
            client.println("<body><h3>Remote ESP32 PWM Control</h3>");
            client.println("<p><a href=\"/off\"><button class=\"button\">Off</button></a>");
            client.println(" <a href=\"/dcdown\"><button class=\"button\">DC -</button></a>");
            client.println(" <a href=\"/dcup\"><button class=\"button\">DC +</button></a>");
            client.println(" <a href=\"/on\"><button class=\"button\">On</button></a></p>");
            client.println("<p><a href=\"/35\"><button class=\"button\">35</button></a>");
            client.println(" <a href=\"/50\"><button class=\"button\">50</button></a>");
            client.println(" <a href=\"/60\"><button class=\"button\">65</button></a>");
            client.println(" <a href=\"/75\"><button class=\"button\">75</button></a></p>");
            client.println(" Duty cycle is set to ");
            client.println(dutyCycle);
            client.println("/255</p>");

            client.println("</body></html>");
            // End the response to client request
            client.println();
            break;
          } else { // if you got a newline, then clear currentLine
            currentLine = "";
          }
        } else if (c != '\r')
        {
          currentLine += c;
        }
      }
    }
    request = "";
    // Close connection to client
    client.stop();
    Serial.println("Disconnected.");
  }
}
