// Project 58 - Using Telegram for Four-GPIO Remote Control

#include <WiFi.h>
#include <WiFiClientSecure.h>
#include <UniversalTelegramBot.h>
#include <ArduinoJson.h>

// Enter your Wi-Fi network SSID and password
const char* ssid = "SSID";
const char* password = "pasword";

// Enter your Telegram bot token and chat ID
#define botToken "token"
#define chatID "chat ID"

#define LED1 17
#define LED2 16
#define LED3 27
#define LED4 14

WiFiClientSecure client;
UniversalTelegramBot bot(botToken, client);

int bot_delay = 1000;
unsigned long lastRun;

void processMessages(int numNewMessages) {
  Serial.println("Handling New Message");
  Serial.println(String(numNewMessages));

  for (int i = 0; i < numNewMessages; i++) {
    // Chat id of the requester
    String chat_id = String(bot.messages[i].chat_id);
    if (chat_id != chatID) {
      bot.sendMessage(chat_id, "Unauthorized user", "");
      continue;
    }

    // Print the received message
    String user_text = bot.messages[i].text;
    Serial.println(user_text);

    String your_name = bot.messages[i].from_name;

    if (user_text == "/start") {
      String startMessage = "Hello, " + your_name + ".\n";
      startMessage += "Choose from the following commands:\n";
      startMessage += "(replace x with LED number 1~4)\n";
      startMessage += "Send /xon to turn LEDx ON \n";
      startMessage += "Send /xoff to turn LEDx ON  \n";
      startMessage += "Send /alloff to turn all LEDs off  \n";
      startMessage += "Send /status to check LED states \n";
      bot.sendMessage(chat_id, startMessage, "");
    }

    if (user_text == "/1on") {
      bot.sendMessage(chat_id, "LED 1 turned on", "");
      digitalWrite(LED1, HIGH);
    }

    if (user_text == "/1off") {
      bot.sendMessage(chat_id, "LED 1 turned off", "");
      digitalWrite(LED1, LOW);
    }

        if (user_text == "/2on") {
      bot.sendMessage(chat_id, "LED 2 turned on", "");
      digitalWrite(LED2, HIGH);
    }

    if (user_text == "/2off") {
      bot.sendMessage(chat_id, "LED 2 turned off", "");
      digitalWrite(LED2, LOW);
    }

    if (user_text == "/3on") {
      bot.sendMessage(chat_id, "LED 3 turned on", "");
      digitalWrite(LED3, HIGH);
    }

    if (user_text == "/3off") {
      bot.sendMessage(chat_id, "LED 3 turned off", "");
      digitalWrite(LED3, LOW);
    }

        if (user_text == "/4on") {
      bot.sendMessage(chat_id, "LED 4 turned on", "");
      digitalWrite(LED4, HIGH);
    }

    if (user_text == "/4off") {
      bot.sendMessage(chat_id, "LED 4 turned off", "");
      digitalWrite(LED4, LOW);
    }

    if (user_text == "/alloff") {
      bot.sendMessage(chat_id, "Turning all LEDs off", "");
      digitalWrite(LED1, LOW);
      digitalWrite(LED2, LOW);
      digitalWrite(LED3, LOW);
      digitalWrite(LED4, LOW);
    }    

    if (user_text == "/status") 
    {
      if (digitalRead(LED1)) 
      {
        bot.sendMessage(chat_id, "LED1 is on", "");
      } else 
      {
        bot.sendMessage(chat_id, "LED1 is off", "");
      }
      if (digitalRead(LED2)) 
      {
        bot.sendMessage(chat_id, "LED2 is on", "");
      } else 
      {
        bot.sendMessage(chat_id, "LED2 is off", "");
      }
      if (digitalRead(LED3)) 
      {
        bot.sendMessage(chat_id, "LED3 is on", "");
      } else 
      {
        bot.sendMessage(chat_id, "LED3 is off", "");
      }
      if (digitalRead(LED4)) 
      {
        bot.sendMessage(chat_id, "LED4 is on", "");
      } else 
      {
        bot.sendMessage(chat_id, "LED4 is off", "");
      }                  
    }
  }
}

void setup() {
  Serial.begin(115200);

  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
  pinMode(LED3, OUTPUT);
  pinMode(LED4, OUTPUT);

  digitalWrite(LED1, LOW);
  digitalWrite(LED2, LOW);
  digitalWrite(LED3, LOW);
  digitalWrite(LED4, LOW);

  // Connect to Wi-Fi
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  client.setCACert(TELEGRAM_CERTIFICATE_ROOT);  // Add root certificate for api.telegram.org
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting to Wi-Fi...");
  }
  Serial.println(WiFi.localIP()); // Display IP address used by ESP32
}

void loop() {
  if (millis() > lastRun + bot_delay) 
  {
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    while (numNewMessages) {
      Serial.println("Received message");
      processMessages(numNewMessages);
      numNewMessages = bot.getUpdates(bot.last_message_received + 1);
    }
    lastRun = millis();
  }
}