// Project 59 - Using Telegram for Remote Data Retrieval

#include <WiFi.h>
#include <WiFiClientSecure.h>
#include <UniversalTelegramBot.h>
#include <ArduinoJson.h>
#include <Adafruit_BMP085.h>

Adafruit_BMP085 bmp;

// Enter your Wi-Fi network SSID and password
const char* ssid = "SSID";
const char* password = "password";

// Enter your Telegram bot token and chatID
#define botToken "5629865778:AAEEkM9pjB2a9Xoky4y6IJxsmVJRp12siYc"
#define chatID "878761727"

WiFiClientSecure client;
UniversalTelegramBot bot(botToken, client);

// Checks for new messages every 1 second.
int bot_delay = 1000;
unsigned long lastRun;

void processMessages(int numNewMessages) {
  String welcome;   // Used for assembling messages to send
  Serial.println("Handling New Message");
  Serial.println(String(numNewMessages));

  for (int i = 0; i < numNewMessages; i++) {
    // Chat id of the requester
    String chat_id = String(bot.messages[i].chat_id);
    if (chat_id != chatID) {
      bot.sendMessage(chat_id, "Unauthorized user", "");
      continue;
    }

    // Print the received message
    String user_text = bot.messages[i].text;
    Serial.println(user_text);

    String your_name = bot.messages[i].from_name;

    if (user_text == "/start") {
      welcome = "Hello, " + your_name + ".\n";
      welcome += "Choose from the following commands:\n\n";
      welcome += "Send /temp for temperature\n";
      welcome += "Send /pressure for air pressure\n";
      welcome += "Send /altitude for altitude\n";
      bot.sendMessage(chat_id, welcome, "");
    }

    if (user_text == "/temp") {
      // get BMP180 temperature
      welcome = "Temperature (C): ";
      welcome += String(bmp.readTemperature());
      bot.sendMessage(chat_id, welcome, "");
    }

    if (user_text == "/pressure") {
      // get BMP180 air pressure, convert to hPa
      welcome = "Air pressure (hPa): ";
      welcome += String(bmp.readSealevelPressure() / 100);
      bot.sendMessage(chat_id, welcome, "");
    }

    if (user_text == "/altitude") {
      // get BMP180 altitude
      welcome = "Altitude (m): ";
      welcome += String(bmp.readAltitude());
      bot.sendMessage(chat_id, welcome, "");
    }
  }
}

void setup() {
  bmp.begin();
  Serial.begin(115200);

  // Connect to Wi-Fi
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  client.setCACert(TELEGRAM_CERTIFICATE_ROOT);  // Add root certificate for api.telegram.org
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting to Wi-Fi...");
  }
  // Print ESP32 Local IP Address
  Serial.println(WiFi.localIP());
}

void loop() {
  if (millis() > lastRun + bot_delay) {
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    while (numNewMessages) {
      Serial.println("Received message");
      processMessages(numNewMessages);
      numNewMessages = bot.getUpdates(bot.last_message_received + 1);
    }
    lastRun = millis();
  }
}