// Project #61 - Retrieving Internet Time

#include <NTPClient.h>
#include <WiFi.h>
#include <WiFiUdp.h>

// Enter your Wi-Fi network SSID and password
const char* ssid = "SSID";
const char* password = "password";
WiFiUDP ntpUDP;

// ntpUDP, time server pool, offset in seconds, update interval (mS)
NTPClient timeClient(ntpUDP, "0.au.pool.ntp.org", 36000, 60000);

void setup() {
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.print("Connecting to Wi-Fi ");
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  timeClient.begin();
}

void loop() {

  String dateString;
  String date, month, year;

  timeClient.update();
  Serial.println(timeClient.getEpochTime());      // Returns an unsigned long
  Serial.println(timeClient.getFormattedDate());  // Returns everything in one string "2023-01-07T15:49:41Z"
  Serial.println(timeClient.getFormattedTime());  // Displays hh:mm:ss

  // Retrieve and display day of week
  Serial.print("Today is ");
  switch (timeClient.getDay()) {
    case 0:
      Serial.println("Sunday");
      break;
    case 1:
      Serial.println("Monday");
      break;
    case 2:
      Serial.println("Tuesday");
      break;
    case 3:
      Serial.println("Wednesday");
      break;
    case 4:
      Serial.println("Thursday");
      break;
    case 5:
      Serial.println("Friday");
      break;
    case 6:
      Serial.println("Saturday");
      break;
    case 7:
      Serial.println("Sunday");
      break;
  }

  // Retrieve hours, minutes and seconds separately and display
  if (timeClient.getHours() < 10) { Serial.print("0"); }
  Serial.print(timeClient.getHours());
  Serial.print("-");
  if (timeClient.getMinutes() < 10) { Serial.print("0"); }
  Serial.print(timeClient.getMinutes());
  Serial.print("-");
  if (timeClient.getSeconds() < 10) { Serial.print("0"); }
  Serial.println(timeClient.getSeconds());

  // Extract data from formatted date result and display
  dateString = timeClient.getFormattedDate();
  date = dateString.substring(8, 10);
  Serial.print(date);
  Serial.print("/");
  month = dateString.substring(5, 7);
  Serial.print(month);
  Serial.print("/");
  year = dateString.substring(0, 4);
  Serial.println(year);
  Serial.println();
  delay(1000);
}