// Project #62 - Creating an OLED Display NTP Clock

#include <NTPClient.h>
#include <WiFi.h>
#include <WiFiUdp.h>
#include <U8g2lib.h>

#include <Wire.h>
U8G2_SSD1306_128X32_UNIVISION_F_HW_I2C u8g2(U8G2_R0, U8X8_PIN_NONE);

// Enter your Wi-Fi network SSID and password
const char* ssid = "SSID";
const char* password = "password";
WiFiUDP ntpUDP;

// ntpUDP, time server pool, offset in seconds, update interval (mS)
NTPClient timeClient(ntpUDP, "0.us.pool.ntp.org", 36000, 60000);

void setup() {
  Wire.begin();
  u8g2.begin();
  u8g2.setFont(u8g2_font_9x18_tr);

  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.print("Connecting to Wi-Fi ");
  while (WiFi.status() != WL_CONNECTED) {
    u8g2.clearBuffer();
    u8g2.drawStr(0, 16, "Connecting");   // need the extra bit c.str to display string on OLED
    u8g2.drawStr(0, 31, "to Wi-Fi...");  // need the extra bit c.str to display string on OLED
    u8g2.sendBuffer();
  }
  timeClient.begin();
}

void loop() 
{
  int hours, minutes, seconds;
  int date, month, year, dayOfWeek;
  String hh, mm, ss, dateString;
  String OLED1, OLED2;

  timeClient.update();
  
  // Assemble time 
  hours = timeClient.getHours();
  minutes = timeClient.getMinutes();
  seconds = timeClient.getSeconds();

  if (hours < 10) { hh = hh + "0"; }
  if (minutes < 10) { mm = mm + "0"; }
  if (seconds < 10) { ss = ss + "0"; }
  hh = hh + String(hours);
  mm = mm + String(minutes);
  ss = ss + String(seconds);
  OLED1 = hh + ":" + mm + ":" + ss;

  // Assemble date
  dateString = timeClient.getFormattedDate();
  switch (timeClient.getDay()) {
    case 0: OLED2 = OLED2 + "Sun "; break;
    case 1: OLED2 = OLED2 + "Mon "; break;
    case 2: OLED2 = OLED2 + "Tue "; break;
    case 3: OLED2 = OLED2 + "Wed "; break;
    case 4: OLED2 = OLED2 + "Thu "; break;
    case 5: OLED2 = OLED2 + "Fri "; break;
    case 6: OLED2 = OLED2 + "Sat "; break;
    case 7: OLED2 = OLED2 + "Sun "; break;
  }

  OLED2 = OLED2 + dateString.substring(8, 10); // Date
  OLED2 = OLED2 + "/";
  OLED2 = OLED2 + dateString.substring(5, 7); // Month
  OLED2 = OLED2 + "/";
  OLED2 = OLED2 + dateString.substring(0, 4); // Year

  // Show time and date on OLED
  u8g2.clearBuffer();
  u8g2.drawStr(0, 16, OLED1.c_str());
  u8g2.drawStr(0, 31, OLED2.c_str());
  u8g2.sendBuffer();
  delay(1000);
}