// Project #64 - Building a Giant NTP LED Clock

#include <NTPClient.h>
#include <WiFi.h>
#include <WiFiUdp.h>

// Enter your Wi-Fi network SSID and password
const char* ssid = "SSID";
const char* password = "password";
WiFiUDP ntpUDP;

// ntpUDP, time server pool, offset in seconds, update interval (mS)
// Starts with home offset (e.g. 36000 for UTC + 10)
NTPClient timeClient(ntpUDP, "0.us.pool.ntp.org", 36000, 60000);

#define latch 5   // Latch RCK pin
#define clock 18  // Clock SRCK pin
#define data 23   // Data SERIN pin

int digits[] = { B00111111,    // 0
                 B00000110,    // 1
                 B01011011,    // 2
                 B01001111,    // 3
                 B01100110,    // 4
                 B01101101,    // 5
                 B01111101,    // 6
                 B00000111,    // 7
                 B01111111,    // 8
                 B01100111 };  // 9

void sendNumbers(int numbers[], int dp) {
  digitalWrite(latch, LOW);
  for (int i = 0; i < 4; i++) {
    int dig_idx = numbers[i];
    if (dp == i) {
      // Display the digit
      shiftOut(data, clock, MSBFIRST, digits[dig_idx] | B10000000);
    } else {
      shiftOut(data, clock, MSBFIRST, digits[dig_idx]);
    }
  }
  digitalWrite(latch, HIGH);
}

void setup() {
  pinMode(latch, OUTPUT);
  pinMode(clock, OUTPUT);
  pinMode(data, OUTPUT);
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.print("Connecting to Wi-Fi ");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }
  timeClient.begin();
}

void loop() {
  int numbers[4];
  int hours, minutes;
  timeClient.update();

  hours = timeClient.getHours();
  minutes = timeClient.getMinutes();

  numbers[3] = hours / 10;
  numbers[2] = hours % 10;
  numbers[1] = minutes / 10;
  numbers[0] = minutes % 10;

  sendNumbers(numbers, 2);
  delay(1000);
}