// Project #65: Logging Time and Temperature Data to Google Sheets

#include <WiFi.h>
#include <HTTPClient.h>
#include <NTPClient.h>
#include <WiFiUdp.h>
#include <Adafruit_BMP085.h>

const char* ssid = "SSID";
const char* password = "password";
WiFiUDP ntpUDP;
Adafruit_BMP085 bmp;

// ntpUDP, time server pool, offset in seconds, update interval (mS)
NTPClient timeClient(ntpUDP, "0.au.pool.ntp.org", 36000, 60000);

String webAppURL = "INSERT WEB APP URL HERE";

unsigned long lastUpdate = 0;
unsigned long updatePeriod = 20000;  // Every 20 seconds

void setup() {
  bmp.begin();
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.print("Connecting to Wi-Fi ");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }
  timeClient.begin();
}

void loop() {
  String dateString, finalAppURL;
  String date, month, year;
  String hh, mm, ss;
  String temperature, pressure;

  if ((millis() - lastUpdate) > updatePeriod) {
    if (WiFi.status() == WL_CONNECTED)  
    {
      HTTPClient http;

      timeClient.update();

      // Extract date from NTP server
      dateString = timeClient.getFormattedDate();
      date = dateString.substring(8, 10);
      month = dateString.substring(5, 7);
      year = dateString.substring(0, 4);

      // Extract time from NTP server
      hh = String(timeClient.getHours());
      mm = String(timeClient.getMinutes());
      ss = String(timeClient.getSeconds());

      // Add time and date to web app URL
      finalAppURL = webAppURL + "?date=" + date + "&month=" + month + "&year=" + year + "&hour=" + hh + "&minute=" + mm + "&second=" + ss;

      // Get weather data from BMP180
      temperature = String(bmp.readTemperature());
      pressure = String(bmp.readSealevelPressure());

      // Add weather data to web app URL
      finalAppURL = finalAppURL + "&temperature=" + temperature + "&airpressure=" + pressure;

      http.begin(finalAppURL.c_str());  // Make HTTP call with webapp URL and data

      int httpCode = http.GET(); // Get response code from server
      if (httpCode > 0) {
        Serial.print("HTTP Response code: ");
        Serial.println(httpCode);
        String httpMessage = http.getString();
        Serial.println(httpMessage);
      } else {
        Serial.print("Error: ");
        Serial.println(httpCode);
      }
      http.end();
    } else {
      Serial.println("Wi-Fi problem.");
    }
    lastUpdate = millis();
  }
}