// Project 66 - Creating a Basic Web Page Server

#include <WiFi.h>
WiFiServer server(80);  // Set webserver port to 80

char* ssid = "SSID";
char* password = "password";

String request;      // Stores HTTP request from client (web browser)

unsigned long currentTime = millis();
unsigned long previousTime = 0;
const long timeoutTime = 2000;  // Allowed client connection time

void setup() {
  Serial.begin(115200);

  // Connect to Wi-Fi network
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED)
  // Show indicator while waiting for connection
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println();  // Display local IP address on serial monitor
  Serial.print("Connected, IP address: ");
  Serial.println(WiFi.localIP());
  //  Start web server
  server.begin();
}

void loop() {
  String millisValue;
  String textLine = "";
  // Listen for incoming clients fromn web browser
  WiFiClient client = server.available();
  if (client)  // If a request received
  {
    currentTime = millis();
    previousTime = currentTime;
    Serial.println("New client connected");
    String currentLine = "";
    while (client.connected() && currentTime - previousTime <= timeoutTime) {  // Stay connected for timeoutTime
      currentTime = millis();
      if (client.available()) {
        char c = client.read();  // Display data from client on Serial Monitor
        Serial.write(c);
        request += c;
        if (c == '\n')  // Client request has finished
        {
          if (currentLine.length() == 0) {
            // Send HTTP reponse back to client
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();

            // Build web page for display on browser
            client.println("<!DOCTYPE html><html><body>");

            // Now HTML for your page
            client.println("<h1>millis() server</h1>");
            millisValue = String(millis());
            textLine += "<p>ESP32 board uptime is : ";
            textLine += millisValue;
            textLine += " milliseconds</p>";
            client.println(textLine);

            client.println("</body></html>");
            // End the response to client request
            client.println();
            break;
          } else {  // If you received a newline, clear currentLine
            currentLine = "";
          }
        } else if (c != '\r') {
          currentLine += c;
        }
      }
    }
    request = "";
    // Close connection to client
    client.stop();
    Serial.println("Disconnected.");
  }
}
