// Project 67 - Creating an ESP32 I/O Port Monitor

#include <WiFi.h>
WiFiServer server(80);  // Set webserver port to 80

char* ssid = "SSID";
char* password = "password";

String request;  // Stores HTTP request from client (web browser)

unsigned long currentTime = millis();
unsigned long previousTime = 0;
const long timeoutTime = 2000;  // Allowed client connection time

void setup() {
  Serial.begin(115200);

  pinMode(18, INPUT);
  pinMode(19, INPUT);
  pinMode(23, INPUT);
  pinMode(5, INPUT);

  // Connect to Wi-Fi network
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED)
  // Show indicator while waiting for connection
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println();  // Display local IP address on serial monitor
  Serial.print("Connected, IP address: ");
  Serial.println(WiFi.localIP());
  //  Start web server
  server.begin();
}

void loop() {
  String textLine = "";
  int IO18, IO19, IO23, IO5;
  float IO2, IO4, IO35, IO34;
  float IO2A, IO4A, IO35A, IO34A;
  String IO2S, IO4S, IO35S, IO34S;
  // Listen for incoming clients fromn web browser
  WiFiClient client = server.available();
  if (client)  // If a request received
  {
    currentTime = millis();
    previousTime = currentTime;
    Serial.println("New client connected");
    String currentLine = "";
    while (client.connected() && currentTime - previousTime <= timeoutTime) {  // Stay connected for timeoutTime
      currentTime = millis();
      if (client.available()) {
        char c = client.read();  // Display data from client on Serial Monitor
        Serial.write(c);
        request += c;
        if (c == '\n')  // Client request has finished
        {
          if (currentLine.length() == 0) {
            // Send HTTP reponse back to client
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();

            // Build web page for display on browser. 
            client.println("<!DOCTYPE html><html><body>");

            // Now HTML for your page
            client.println("<h1>ESP32 I/O Monitor</h1>");
            client.println("<p> Status as of last update. Refresh browser for latest values.</p>");
            client.println("<hr>");

            IO18 = digitalRead(18);
            IO19 = digitalRead(19);
            IO23 = digitalRead(23);
            IO5 = digitalRead(5);

            IO2 = analogRead(2);
            IO4 = analogRead(4);
            IO35 = analogRead(35);
            IO34 = analogRead(34);

            IO2A = ((IO2 / 4096) * 3.3);
            IO4A = ((IO4 / 4096) * 3.3);
            IO35A = ((IO35 / 4096) * 3.3);
            IO34A = ((IO34 / 4096) * 3.3);

            client.println("<p><b>Digital Input Status:</b></p>");
            textLine += "<p>Digital pin IO18 is ";
            if (IO18 == 1) {
              textLine += "HIGH";
            } else {
              textLine += "LOW</p>";
            }
            client.println(textLine);
            textLine = "";
            textLine += "<p>Digital pin IO19 is ";
            if (IO19 == 1) {
              textLine += "HIGH";
            } else {
              textLine += "LOW</p>";
            }
            client.println(textLine);
            textLine = "";
            textLine += "<p>Digital pin IO23 is ";
            if (IO23 == 1) {
              textLine += "HIGH";
            } else {
              textLine += "LOW</p>";
            }
            client.println(textLine);
            textLine = "";
            textLine += "<p>Digital pin IO5 is ";
            if (IO5 == 1) {
              textLine += "HIGH";
            } else {
              textLine += "LOW</p>";
            }
            client.println(textLine);
            textLine = "";
            client.println("");
            client.println("<p><b>Analog Input Status:</b></p>");
            textLine += "Analog pin IO2 raw value: ";
            textLine += String(IO2);
            textLine += "; Voltage (V): ";
            client.println(textLine);
            client.println(IO2A);
            client.println("</p>");
            textLine = "";

            textLine += "Analog pin IO4 raw value: ";
            textLine += String(IO4);
            textLine += "; Voltage (V): ";
            client.println(textLine);
            client.println(IO4A);
            client.println("</p>");
            textLine = "";

            textLine += "Analog pin IO35 raw value: ";
            textLine += String(IO35);
            textLine += "; Voltage (V): ";
            client.println(textLine);
            client.println(IO35A);
            client.println("</p>");
            textLine = "";

            textLine += "Analog pin IO34 raw value: ";
            textLine += String(IO34);
            textLine += "; Voltage (V): ";
            client.println(textLine);
            client.println(IO34A);
            client.println("</p>");
            textLine = "";
            client.println("<hr>");

            client.println("</body></html>");
            // End the response to client request
            client.println();
            break;
          } else {  // If you received a newline, clear currentLine
            currentLine = "";
          }
        } else if (c != '\r') {
          currentLine += c;
        }
      }
    }
    request = "";
    // Close connection to client
    client.stop();
    Serial.println("Disconnected.");
  }
}
