// Project 68 - Creating an ESP32 Time and Weather Server

#include <Adafruit_BMP085.h>
Adafruit_BMP085 bmp;
#include <NTPClient.h>
#include <WiFi.h>
#include <WiFiUdp.h>
WiFiUDP ntpUDP;
WiFiServer server(80);  // Set webserver port to 80

char* ssid = "SSID";
char* password = "password";

// ntpUDP, time server pool, offset in seconds, update interval (mS)
NTPClient timeClient(ntpUDP, "0.au.pool.ntp.org", 36000, 60000);

String request;  // Stores HTTP request from client (web browser)

unsigned long currentTime = millis();
unsigned long previousTime = 0;
const long timeoutTime = 2000;  // Allowed client connection time

void setup() {
  Serial.begin(115200);
  bmp.begin();
  // Connect to Wi-Fi network
  WiFi.mode(WIFI_STA);  
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED)
  // Show indicator while waiting for connection
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println();  // Display local IP address on serial monitor
  Serial.print("Connected, IP address: ");
  Serial.println(WiFi.localIP());
  //  Start web server
  server.begin();
}

void loop() {
  String dateString;
  String date, month, year;

  // Listen for incoming clients fromn web browser
  WiFiClient client = server.available();
  if (client)  // If a request received
  {
    currentTime = millis();
    previousTime = currentTime;
    Serial.println("New client connected");
    String currentLine = "";
    while (client.connected() && currentTime - previousTime <= timeoutTime) {  // Stay connected for timeoutTime
      currentTime = millis();
      if (client.available()) {
        char c = client.read();  // Display data from client on Serial Monitor
        Serial.write(c);
        request += c;
        if (c == '\n')  // Client request has finished
        {
          if (currentLine.length() == 0) {
            // Send HTTP reponse back to client
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();

            // Build web page for display on browser
            client.println("<!DOCTYPE html><html><body>");
            client.println("<h1>ESP32 Time and Weather Server</h1>");
            client.println("<p>Refresh browser for latest information.</p>");
            client.println("<hr>");
            client.print("<p>Time: ");
            timeClient.update();
            client.print(timeClient.getFormattedTime());  // Displays hh:mm:ss
            client.print("</p>");
            client.print("<p>Today is ");
            switch (timeClient.getDay()) {
              case 0: client.print("Sunday "); break;
              case 1: client.print("Monday "); break;
              case 2: client.print("Tuesday "); break;
              case 3: client.print("Wednesday "); break;
              case 4: client.print("Thursday "); break;
              case 5: client.print("Friday "); break;
              case 6: client.print("Saturday "); break;
              case 7: client.print("Sunday "); break;
            }
            // Extract data from formatted date result and display
            dateString = timeClient.getFormattedDate();
            date = dateString.substring(8, 10);
            client.print(date);
            client.print("/");
            month = dateString.substring(5, 7);
            client.print(month);
            client.print("/");
            year = dateString.substring(0, 4);
            client.print(year);
            client.print("</p>");
            client.println("<hr>");
            client.print("<p>Current temperature (C): ");
            client.print(bmp.readTemperature());
            client.print("</p>");
            client.print("<p>Current air pressure at sea level (calculated) (Pa): ");
            client.print(bmp.readSealevelPressure());
            client.print("</p>");
            client.println("<hr>");
            client.println("</body></html>");
            // End the response to client request
            client.println();
            break;
          } else {  // If you received a newline, clear currentLine
            currentLine = "";
          }
        } else if (c != '\r') {
          currentLine += c;
        }
      }
    }
    request = "";
    // Close connection to client
    client.stop();
    Serial.println("Disconnected.");
  }
}
